/* -*- c++ -*- */
/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

#ifndef MPS_Transport_H
#define MPS_Transport_H

namespace MPS {

/**
 * Represents a method of resolving names for, and connecting to
 * remote objects. Subclasses implement name-resolution and connection
 * facilities according to a particular transport protocol; examples
 * include InetTransport, SleeTransport, and FastmsgTransport. */
class Transport {
private:
  typedef vector<Transport *> transportVec_t;		///< A list of Transports.
  static transportVec_t allTransports;			///< All registered Transports

private:
  string name;				///< Registered name of this transport ("inet", "slee", ..)

private:
  /**
   * Locate a Transport by name.
   *
   * @return a pointer to the named Transport.
   * @exception MPSException if the Transport name is not registered. */
  static Transport *findTransport(string const &name);

public:
  /// Register a transport.
  static void registerTransport(Transport *t);

  /**
   * Returns a message source/sink that can be used for communication
   * with the object at the address described by the
   * connectionSpec. Selects an appropriate Transport based on the
   * "transport" field of the Address.
   *
   * @param connectionSpec the address of the remote object
   * @return a connection handle for remote communication, or NULL if
   * none could be created
   * @exception MPSException if the transport name given is not
   * registered */
  static ref<Connection> getConnectionTo(Address const &connectionSpec);

  /**
   * Bind an object server to an interface provided by a transport.
   *
   * @param server the server to bind to the (about-to-be-created) transport-interface
   *
   * @param bindingSpec the suggested resolved name to bind to -
   * should be of the form "mps:transportname" or, if a greater degree
   * of control over the bind is desired,
   * "mps:transportname:param:param:..." (for instance, with the
   * "inet" transport, you use "mps:inet:hostname:portnumber").
   *
   * @return the string of the newly-bound address, or empty-string if the bind failed */
  static string bindServer(Server *server, string const &bindingSpec);

  /**
   * Bind an object server to all available transports that currently
   * do not have a binding for this server.
   *
   * @param server the server to bind */
  static void bindServer(Server *server);

  /**
   * Unbind an object server from an interface provided by a transport.
   *
   * @param server the server to unbind to the (about-to-be-removed) transport-interface
   *
   * @param bindingSpec the suggested resolved name to unbind from - cf. bindServer()
   * @return the string of the newly-unbound address, or empty-string if the unbind failed */
  static string unbindServer(Server *server, string const &bindingSpec);

  /**
   * Unbind a previously-bound server.
   * @param server the server to unbind. */
  static void unbindServer(Server *server);

protected:
  /**
   * For access by subclasses - initialises this class
   *
   * @param _name name to register the transport under
   */
  Transport(string const &_name);

public:
  virtual ~Transport();

  /// Retrieves the transport's registered name.
  string const &getName() const;

protected:
  /**
   * Returns a message source/sink that can be used for communication
   * with the object at the address described by the
   * connectionSpec. The connectionSpec is the broken-down resolved
   * name of the remote object, and <b>must</b> match the address
   * format that this transport expects.
   *
   * @param connectionSpec the address of the remote object
   * @return a connection handle for remote communication, or NULL if
   * none could be created */
  virtual ref<Connection> connectTo(Address const &connectionSpec) = 0;

  /**
   * Attempt to register a server interface for access using this transport.
   *
   * @param server the server to register
   * @param spec the connection specification that the transport may
   * use as a hint in determining the final resolved name to use when
   * binding this object.
   * @return the canonical name of the server-bound-to-this-transport,
   * or "" if binding to this transport fails. */
  virtual string registerServer(Server *server, Address const &spec) = 0;

  /**
   * Attempt to deregister a server interface.
   *
   * @param server the server to deregister
   * @param spec the connection specification that the transport may
   * use as a hint in determining the final resolved name to use when
   * unbinding this object.
   * @return the canonical name of the server-that-was-bound-to-this-transport,
   * or "" if unbinding from this transport fails. */
  virtual string deregisterServer(Server *server, Address const &spec) = 0;
};

}

#endif
